package controller

import (
	"context"
	"errors"
	"fonchain-artshow/cmd/service"
	"fonchain-artshow/pb/test"
	"fonchain-artshow/pkg/m"
)

func (p *ArtShowProvider) CreateApply(ctx context.Context, req *test.SaveApplyReq) (res *test.SaveApplyRes, err error) {
	if req.Applicant == "" {
		err = errors.New(m.ERROR_APPLICANT)
		return
	}
	if req.ApplyTime == "" {
		err = errors.New(m.ERROR_TIME)
		return
	}
	res = new(test.SaveApplyRes)
	err, applyID := service.CreateShowApply(req)
	if err != nil {
		res.Msg = err.Error()
		err = errors.New(m.ERROR_CREATE)
		return
	}
	res.Msg = m.CREATE_SUCCESS
	res.ApplyID = int64(applyID)
	return
}

func (p *ArtShowProvider) UpdateApply(ctx context.Context, req *test.SaveApplyReq) (res *test.SaveApplyRes, err error) {
	if req.ID == 0 {
		err = errors.New(m.ERROR_INVALID_ID)
		return
	}
	res = new(test.SaveApplyRes)
	err, showID := service.UpdateShowApplyWithShowRel(req)
	if err != nil {
		res.Msg = err.Error()
		err = errors.New(m.UPDATE_FAILED)
		return
	}
	res.Msg = m.UPDATE_SUCCESS
	res.ApplyID = int64(showID)
	return
}

func (p *ArtShowProvider) ApplyList(ctx context.Context, req *test.ApplyListReq) (res *test.ApplyListRes, err error) {
	res = new(test.ApplyListRes)
	if req.Page == 0 {
		req.Page = 1
	}
	if req.PageSize == 0 {
		req.Page = 10
	}
	err, res.Total, res.Data = service.ShowApplyList(req)
	if err != nil {
		res.Msg = err.Error()
		err = errors.New(m.ERROR_QUERY)
		return
	}
	return
}

func (p *ArtShowProvider) ApplyDetail(ctx context.Context, req *test.ApplyShowReq) (res *test.ApplyShowRes, err error) {
	if req.ApplyID == 0 {
		err = errors.New(m.ERROR_INVALID_ID)
		return
	}
	res = new(test.ApplyShowRes)
	err, res = service.ShowApplyDetail(uint(req.ApplyID))
	if err != nil {
		res.Msg = err.Error()
		err = errors.New(m.ERROR_QUERY)
		return
	}
	return
}

func (p *ArtShowProvider) DelApply(ctx context.Context, req *test.DelApplyReq) (res *test.CommonRes, err error) {
	if len(req.ApplyID) < 1 {
		err = errors.New(m.ERROR_INVALID_ID)
		return
	}
	res = new(test.CommonRes)
	err = service.DelApplyByApplyID(req)
	if err != nil {
		res.Msg = err.Error()
		err = errors.New(m.ERROR_DELETE)
		return
	}
	res.Msg = m.DELETE_SUCCESS
	return
}

func (p *ArtShowProvider) UpdateApplyStatus(ctx context.Context, req *test.UpdateApplyStatusReq) (res *test.CommonRes, err error) {
	if req.ApplyID == 0 {
		err = errors.New(m.ERROR_INVALID_ID)
		return
	}
	res = new(test.CommonRes)
	err = service.UpdateShowApplyStatus(req)
	if err != nil {
		res.Msg = err.Error()
		err = errors.New(m.UPDATE_FAILED)
		return
	}
	res.Msg = m.UPDATE_SUCCESS
	return
}
